// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <array>
#include <memory>
#include <vector>

#include <aoc/extra/ChronospatialComputerInstruction.hpp>
#include <aoc/extra/ChronospatialComputerState.hpp>
#include <aoc/framework/Solver-types.hpp>

class ChronospatialComputer
    : public Solver<std::string, int64_t>
{
public:
    ChronospatialComputer();
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
    void runProgram(const std::vector<int>& program, ChronospatialComputerState& state) const;
private:
    std::array<std::unique_ptr<ChronospatialComputerInstruction>, 8> instructions_;
    std::vector<int> program_;
    ChronospatialComputerState state_;
    int64_t findQuineState();
};
