// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <aoc/common/Point2.hpp>
#include <aoc/framework/Solver-types.hpp>

class ClawContraption
    : public Solver<int64_t, int64_t>
{
public:
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    Point2 buttonA_;
    Point2 buttonB_;
    static constexpr int getNMaxButtonPushes();
    static constexpr int getButtonACost();
    static constexpr int getButtonBCost();
    static constexpr int64_t getConversionCorrection();
    int64_t calcTokenCost(const Point2& buttonA, const Point2& buttonB, const Point2& prize,
        const int64_t offset);
};
