// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <unordered_map>

#include <aoc/extra/KeypadPatternTransformation.hpp>
#include <aoc/extra/KeypadRobot.hpp>
#include <aoc/framework/Solver-types.hpp>

class KeypadConundrum
    : public Solver<int64_t, int64_t>
{
public:
    KeypadConundrum();
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    static constexpr char getStartPositionChar();
    static constexpr size_t getPart1NRobots();
    static constexpr size_t getPart2NRobots();
    KeypadRobot numericKeyboardRobot_;
    KeypadRobot directionalKeyboardRobot_;
    std::unordered_map<std::string, KeypadPatternTransformation> transformations_;
    void updateTransformationMap(KeypadPatternTransformation& targetTransformation);
    void updateTransformationMapLengths(const std::vector<KeypadPatternTransformation*>& added);
    int64_t calcAccumulatedLength(KeypadPatternTransformation& transformation, const size_t index);
};
