// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <map>

#include <aoc/common/Graph.hpp>
#include <aoc/framework/Solver-types.hpp>

class LanParty
    : public Solver<int64_t, std::string>
{
public:
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    static constexpr std::string getFirstTxComputerName();
    static constexpr std::string getLastTxComputerName();
    Graph<std::string, void> lan_{};
    std::map<std::string, int> labelMap_{};
    int findOrAddVertex(const std::string& vertexId);
    void computeInterconnectedThreeSetCount(const int vertexTx);
    bool canProcessVertex(const int vertexToCheck, const int vertexTx) const;
    bool findCompleteSubgraph(const int vertex, const size_t targetSize);
    bool isSubgraphComplete(const std::vector<int>& neighbors, const std::vector<bool>& combination) const;
    std::string calcPassword(const std::vector<int>& neighbors, const std::vector<bool>& combination,
        const int vertex) const;
};
