// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <unordered_map>

#include <aoc/framework/Solver-types.hpp>

class MonkeyMarket
    : public Solver<int64_t, int64_t>
{
public:
    MonkeyMarket(const int inputFileNameSuffix = 0);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    static constexpr int getNSecretNumbers();
    static constexpr uint64_t getPruneValue();
    static constexpr int getPriceModulo();
    uint64_t nBuyers_;
    std::unordered_map<uint64_t, std::pair<uint64_t, int>> sequenceResults_;
    void updateNewRollingChangeSequence(uint64_t& rollingChangeSequence, const uint64_t change);
};
