// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <functional>

#include <aoc/common/Dijkstra.hpp>
#include <aoc/common/Graph.hpp>
#include <aoc/common/Grid.hpp>
#include <aoc/framework/Solver-types.hpp>

class RamRun
    : public Solver<int64_t, std::string>
{
public:
    RamRun(const size_t memorySize = 71, const int maxBytes = 1024);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    size_t memorySize_;
    int maxBytes_;
    int nBytes_;
    Grid<int> vertexReferences_;
    std::function<int(int)> edgeWeightFunctor_;
    Dijkstra::PathsResult dijkstraResult_;
    static constexpr char getUnknownVertexReference();
    static constexpr char getNoVertexReference();
    bool tryMarkCorrupted(const std::string& line, int& corruptedVertex);
    Graph<void, void> buildGraph();
    int getVertex(Graph<void, void>& graph, const size_t x, const size_t y);
    void resetVertexReferences();
};
