// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <list>
#include <map>
#include <set>
#include <vector>

#include <aoc/common/Graph.hpp>
#include <aoc/extra/ReindeerMazeCrossing.hpp>
#include <aoc/extra/ReindeerMazePathIncidence.hpp>
#include <aoc/framework/LinesSolver.hpp>

class ReindeerMaze
    : public LinesSolver<int64_t, int64_t>
{
public:
    ReindeerMaze(const int inputFileNameSuffix = 0);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void finish() override;
private:
    using VertexAttachedPositions = std::map<int, std::pair<int, int>>;
    static constexpr char getStartChar();
    static constexpr char getEndChar();
    static constexpr char getWallChar();
    static constexpr int getTurnCost();
    void buildPathSegmentGraph(Graph<int, int>& graph, VertexAttachedPositions& vertexAttachedPositions,
        const int entry, const int exit);
    void initializeWorkList(std::list<ReindeerMazeCrossing>& crossings, const int entryVertex);
    void addCheckedIncidence(std::vector<ReindeerMazePathIncidence>& incidences, const Point2 start,
        const Point2 direction);
    void addPathSegmentEdges(Graph<int, int>& graph, const ReindeerMazePathIncidence& pathIncidence,
        const std::vector<ReindeerMazePathIncidence>& otherPathIncidences);
    std::pair<int, int> makePositionsIdPair(const Point2& position1, const Point2& position2);
    int calcShortestPaths(const Graph<int, int>& graph, const VertexAttachedPositions& vertexAttachedPositions,
        const int entry, const int exit, const std::vector<int>& shortestDistances);
    int getNUniqueAttachedPositions(const VertexAttachedPositions& vertexAttachedPositions,
        const std::set<int>& vertices);
};
