// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <functional>
#include <vector>

#include <aoc/common/GraphBase.hpp>

class Dijkstra
{
public:
    static constexpr int getInfiniteDistance();

    class PathsResult
    {
    public:
        PathsResult() : distances{}, predecessors{}
        {
        }
        PathsResult(const size_t size, const int initialDistance, const int initialPredecessor)
            : distances(size, initialDistance), predecessors(size, initialPredecessor)
        {
        }
        std::vector<int> distances;
        std::vector<int> predecessors;
    };

    static PathsResult run(const GraphBase& graph, const std::function<int(int)>& edgeWeightFunctor, const int source);
};
