// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <iterator>
#include <vector>
#include <type_traits>

#include <aoc/common/GraphBase.hpp>

template <typename TVertexData = void, typename TEdgeData = void>
class Graph
    : public GraphBase
{
public:
    int addVertex()
        requires(std::is_void_v<TVertexData>)
    {
        return addVertexInternal();
    }

    template <typename T = TVertexData>
        requires(!std::is_void_v<T>)
    int addVertex(const T& data)
    {
        vertexData_.push_back(data);
        return addVertexInternal();
    }

    void addEdge(const int vertex1, const int vertex2)
        requires(std::is_void_v<TEdgeData>)
    {
        addEdgeInternal(vertex1, vertex2);
    }

    template <typename T = TEdgeData>
        requires(!std::is_void_v<T>)
    void addEdge(const int vertex1, const int vertex2, const T& data)
    {
        edgeData_.push_back(data);
        addEdgeInternal(vertex1, vertex2);
    }

    template <typename T = TVertexData>
        requires(!std::is_void_v<T>)
    const T& getVertexData(const int vertex) const
    {
        return vertexData_[vertex];
    }

    template <typename T = TEdgeData>
        requires(!std::is_void_v<T>)
    const T& getEdgeData(int edge) const
    {
        return edgeData_[edge];
    }

private:
    std::conditional_t<std::is_void_v<TVertexData>, char, std::vector<TVertexData>> vertexData_{};
    std::conditional_t<std::is_void_v<TEdgeData>, char, std::vector<TEdgeData>> edgeData_{};
};
