// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <string>

template <typename T1, typename T2>
class Solver
{
public:
    Solver(const int inputFileNameSuffix = 0);
    virtual ~Solver() {};
    const std::string getInputFileName() const;
    virtual const std::string getPuzzleName() const = 0;
    virtual const int getPuzzleDay() const = 0;
    virtual void processDataLine(const std::string& line) = 0;
    virtual void finish() = 0;
    const T1& getResultPart1() const;
    const T2& getResultPart2() const;
protected:
    T1 part1;
    T2 part2;
private:
    int inputFileNameSuffix_;
    std::string clean(const std::string& original) const;
};
