// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/MonkeyMarket.hpp>

MonkeyMarket::MonkeyMarket(const int inputFileNameSuffix)
    : Solver{ inputFileNameSuffix }, nBuyers_{ 0 }, sequenceResults_{}
{
}

const std::string MonkeyMarket::getPuzzleName() const
{
    return "Monkey Market";
}

const int MonkeyMarket::getPuzzleDay() const
{
    return 22;
}

void MonkeyMarket::processDataLine(const std::string& line)
{
    uint64_t n{ std::stoull(line) };
    uint64_t price{ n % getPriceModulo() };
    uint64_t change;
    uint64_t newPrice;
    uint64_t rollingChangeSequence{ 0 };
    for (int i = 0; i < getNSecretNumbers(); i++)
    {
        n = ((n << 6) ^ n) & getPruneValue();
        n = (n >> 5) ^ n;
        n = ((n << 11) ^ n) & getPruneValue();
        newPrice = n % getPriceModulo();
        change = getPriceModulo() + newPrice - price;
        price = newPrice;
        updateNewRollingChangeSequence(rollingChangeSequence, change);
        if (i >= 3)
        {
            const auto& [it, success] = sequenceResults_.insert({ rollingChangeSequence, { newPrice, nBuyers_ } });
            if (!success && it->second.second < nBuyers_)
            {
                it->second.first += newPrice;
                it->second.second = nBuyers_;
                if (it->second.first > part2)
                {
                    part2 = it->second.first;
                }
            }
        }
    }
    part1 += n;
    nBuyers_++;
}

void MonkeyMarket::finish()
{
}

constexpr int MonkeyMarket::getNSecretNumbers()
{
    return 2000;
}

constexpr uint64_t MonkeyMarket::getPruneValue()
{
    return 16777215;
}

constexpr int MonkeyMarket::getPriceModulo()
{
    return 10;
}

void MonkeyMarket::updateNewRollingChangeSequence(uint64_t& rollingChangeSequence, const uint64_t change)
{
    // Prices vary from -9 to 9, so we can store the price change plus the price modulo of 10, which is a number between
    // 1 and 19, as a 5-bit part in this rolling change sequence number and use it as the key in the sequence map.
    rollingChangeSequence = ((rollingChangeSequence << 5) & 0b11111111111111111111) | change;
}
