// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/common/Dijkstra.hpp>

#include <limits>
#include <queue>

constexpr int Dijkstra::getInfiniteDistance()
{
    return std::numeric_limits<int>::max();
}

Dijkstra::PathsResult Dijkstra::run(const GraphBase& graph, const std::function<int(int)>& edgeWeightFunctor,
    const int source)
{
    PathsResult result{ graph.getNVertices(), getInfiniteDistance(), -1 };
    auto compare = [&result](int left, int right) { return result.distances[left] > result.distances[right]; };
    std::priority_queue<int, std::vector<int>, decltype(compare)> queue{ compare };

    result.distances[source] = 0;
    queue.push(source);

    while (!queue.empty())
    {
        int v{ queue.top() };
        queue.pop();

        for (auto neighbor = graph.begin(v); neighbor != graph.end(); ++neighbor)
        {
            int newDistance{ result.distances[v] + edgeWeightFunctor(neighbor->edge) };
            if (result.distances[neighbor->vertex] > newDistance)
            {
                result.distances[neighbor->vertex] = newDistance;
                result.predecessors[neighbor->vertex] = v;
                queue.push(neighbor->vertex);
            }
        }
    }

    return result;
}
