// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/extra/KeypadRobot.hpp>

#include <sstream>

KeypadRobot::KeypadRobot(const std::map<char, Point2>&& keypad, const Point2&& forbidden)
    : keypad_{ keypad }, forbidden_{ forbidden }
{
}

KeypadPatternTransformation KeypadRobot::calcTransformation(const std::string& targetOutputKeys) const
{
    KeypadPatternTransformation result{};
    result.accumulatedLengths.push_back(targetOutputKeys.size());

    std::ostringstream stream{};
    Point2 position{ 0, 0 };
    for (const char c : targetOutputKeys)
    {
        Point2 next = keypad_.at(c);

        // This specific order of robot arm movements aims to reduce resulting combinations of 'A' and '<' for the
        // second robot, which expand to more key presses starting with the third robot.
        bool horizontalFirst{ (next.x < position.x && !isForbidden(next.x, position.y)) ||
            isForbidden(position.x, next.y) };
        if (horizontalFirst)
        {
            move(stream, next.x - position.x, '>', '<', result.nDuplicates);
        }
        move(stream, next.y - position.y, 'v', '^', result.nDuplicates);
        if (!horizontalFirst)
        {
            move(stream, next.x - position.x, '>', '<', result.nDuplicates);
        }
        stream << 'A';

        result.parts.push_back(stream.str());
        stream.str("");

        position = next;
    }

    return result;
}

void KeypadRobot::move(std::ostringstream& stream, const int delta, const char positive, const char negative,
    int& nDuplicates) const
{
    if (delta > 0)
    {
        stream << positive;
        nDuplicates += delta - 1;
    }
    else if (delta < 0)
    {
        stream << negative;
        nDuplicates -= delta + 1;
    }
}

bool KeypadRobot::isForbidden(const int x, const int y) const
{
    return x == forbidden_.x && y == forbidden_.y;
}
